require 'colorize'		# colorful console output
require_relative 'xcode_project'

module Fame
  # Handles import and export of .xliff files
  class XliffImport
    # All accepted XLIFF file types
    ACCEPTED_FILE_TYPES = [".xliff"].freeze

    # All required localization files
    # If these files are not present in their respective .lproj folder
    # the `xcodebuild -importLocalizations` command will fail
    REQUIRED_LOCALIZATION_FILES = ["Localizable.strings", "InfoPlist.strings"].freeze

    #
    # Initializer
    # @param xcode_proj_path A path to a .xcodeproj file whose contents should be localized.
    #
    def initialize(xcode_proj_path)
      @xcode_proj = XcodeProject.new(xcode_proj_path)
    end

    #
    # Imports all .xliff files at the given path into the current Xcode project
    # @param path A folder of .xliff files that should be imported into the current Xcode project.
    #
    def import(path)
      xliffs = determine_xliff_files!(path)
      puts "Found #{xliffs.count} xliff file(s) -> #{xliffs.map { |x| File.basename(x, '.*') }}".light_black

      xliffs.each_with_index do |xliff, index|
        language =  File.basename(xliff, '.*')
        puts "(#{index+1}/#{xliffs.count}) [#{language}] Importing #{xliff}".blue

        `xcodebuild -importLocalizations -localizationPath #{xliff} -project #{@xcode_proj.xcode_proj_path}`
      end
    end

    #
    # Import of .xliff files using `xcodebuild` may fail in case the appropriate
    # .strings file do not already exist.
    # ```
    # xcodebuild: error: Importing localizations from en.xliff will make changes to Example.
    # Import with xcodebuild can only modify existing strings files.
    # Please import with Xcode instead of xcodebuild to make the changes to Example.
    # ```
    #
    # This method creates the missing localization .strings files in order to silence this error.
    #
    def create_missing_strings_files
      # search for Base.lproj folder in project
      project_root = File.expand_path("..", @xcode_proj.xcode_proj_path)
      base_lproj = Dir.glob(project_root + "/**/*{Base.lproj}").first # TODO: ask if multiple...
      raise "There is no Base.lproj folder, please make sure to have a base localization" unless base_lproj
      lproj_root = File.expand_path("..", base_lproj)

      # collect all names of files that need to be generated in each <language>.lproj folder
      # TODO: not sure if we should determine the project_root like this or kust use an input param from the cli?
      ib_file_names = InterfaceBuilder.determine_ib_files!(project_root)
        .map { |f| File.basename(f, ".*") }
        .map { |f| "#{f}.strings"}
      required_files = REQUIRED_LOCALIZATION_FILES + ib_file_names

      # Create <language>.lproj files for all languages
      @xcode_proj.all_languages.each do |language|
        # create .lproj folder
        lproj_folder_path = File.join(lproj_root, "#{language}.lproj")
        Dir.mkdir(lproj_folder_path) unless File.exists?(lproj_folder_path)
        puts "Checking for missing .strings files for #{lproj_folder_path}".blue

        required_files.each do |file_name|
          path = File.join(lproj_folder_path, file_name)
          exists = File.exists?(path)
          File.write(path, "") unless exists

          puts "#{file_name}".colorize(exists ? :yellow : :green) + " at #{path}".light_black
        end
      end

      # <ib_file>.strings
      # Localizable.strings
      # InfoPlist.strings

      # search for Base.lproj
      # create .lproj folders if needed
      # create files for each `string_files` entry
    end

    private

    #
    # Searches the given path for .xliff files and returns their paths.
    # @param path The path that should be searched for .xliff files.
    # @return [Array<String>] An array of paths to .xliff files.
    #
    def determine_xliff_files!(path)
      raise "[XliffImport] The provided file or folder does not exist" unless File.exist? path

      if File.directory?(path)
        files = Dir.glob(path + "/**/*{#{ACCEPTED_FILE_TYPES.join(',')}}")
        raise "[XliffImport] The provided folder did not contain any XLIFF files (#{ACCEPTED_FILE_TYPES.join(', ')})" unless files.count > 0
        return files
      else
        raise "[XliffImport] The provided file is not an XLIFF (#{ACCEPTED_FILE_TYPES.join(', ')})" unless ACCEPTED_FILE_TYPES.include? File.extname(path)
        return [path]
      end
    end

  end
end
